/*
	Author: Silver Moon ( m00n.silv3r@gmail.com )
	Description: Send Raw packets using winpcap
	Using SendArp of iphlpapi.dll to retrive the MAC Address of the IP
	Address. SendArp function available only on Windows 2000 and above
*/

#define HAVE_REMOTE // For Promiscous mode sniffing :)
#define _CRT_SECURE_NO_WARNINGS
#define _WINSOCK_DEPRECATED_NO_WARNINGS
#ifndef WIN32
#define WIN32
#endif
#undef UNICODE
//#undef _WIN64
#include "pcap.h"
#include "raw.h"
#include "adapter.h"

#pragma comment(lib,"ws2_32.lib") //For winsock
#pragma comment(lib,"wpcap.lib") //For winpcap

char *IP_SRC = 0;
#define IP_DEST "192.168.1.57" 

void ifprint(pcap_if_t* d); in_addr getIPAddr(pcap_if_t* d);
//Checksum function
USHORT in_checksum(unsigned short*, int);
int init_winsock();

void mdump(const unsigned char* data_buffer, const unsigned int length) {
	unsigned char byte;
	unsigned int i, j;
	for (i = 0; i < length; i++) {
		byte = data_buffer[i];
		printf("%02x ", data_buffer[i]);  // display byte in hex
		if (((i % 16) == 15) || (i == length - 1)) {
			for (j = 0; j < 15 - (i % 16); j++)
				printf("   ");
			printf("| ");
			for (j = (i - (i % 16)); j <= i; j++) {  // display printable bytes from line
				byte = data_buffer[j];
				if ((byte > 31) && (byte < 127)) // outside printable char range
					printf("%c", byte);
				else
					printf(".");
			}
			printf("\n"); // end of the dump line (each line 16 bytes)
		} // end if
	} // end for
}
int main()
{
	pcap_if_t* alldevs, * d, dev[100];
	pcap_t* fp;
	pcap_addr_t* a;

	u_char packet[65536], s_mac[6], d_mac[6];
	in_addr srcip, destip;

	ETHER_HDR* ehdr;
	IPV4_HDR* iphdr;
	TCP_HDR* tcphdr;
	P_HDR pseudo_header;
	const char* dump = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

	char sgatewayip[16], errbuf[PCAP_ERRBUF_SIZE + 1], * data;
	int gatewayip, count = 1, process_id = GetCurrentProcessId();

	char acUserName[100];
	DWORD nUserName = sizeof(acUserName);

	if (GetUserName(acUserName, &nUserName))
	{
		printf("Username : %s \n", acUserName);
	}
	else
	{
		printf("Failed to lookup user name, error code %d \n", GetLastError());
	}

	loadiphlpapi();

	// Retrieve the interfaces list from winpcap
	printf("Retrieving the available devices...");

	if (pcap_findalldevs_ex((char*)"rpcap://", NULL, &alldevs, errbuf) == -1)
	{
		fprintf(stderr, "Error in pcap_findalldevs: %s\n", errbuf);
		exit(1);
	}
	printf("Retrieved.\n");

	printf("The following devices found : \n\n");
	for (d = alldevs; d; d = d->next)	//Print the devices
	{
		printf("%d)\n", count);
		dev[count++] = *d;
		ifprint(d);
	}

	//Ask user to select the device he wants to use
	printf("Enter the device number you want to use : ");
	scanf("%d", &count);

	a = dev[count].addresses;

	//Get mac addresses of source and gateway ips
	struct sockaddr_in *sain = (struct sockaddr_in*)a->addr;
	srcip = ((struct sockaddr_in*)a->addr)->sin_addr;
	//long x=inet_addr("192.168.1.150");
	in_addr lip = getIPAddr(&dev[count]);
	//char* xa = inet_ntoa(((struct sockaddr_in*)a->addr)->sin_addr);// inet_ntoa(srcip)
	IP_SRC = inet_ntoa(lip);
	srcip = lip;
	//xa = htonl(l);
	GetMacAddress(s_mac, srcip);
	//mdump( reinterpret_cast<const unsigned char *>(&srcip),128);//(const unsigned char *)dev[count].addresses,128);
	printf("Selected device has mac address [%s]: %.2X-%.2X-%.2X-%.2X-%.2X-%.2X", inet_ntoa(srcip), s_mac[0], s_mac[1], s_mac[2], s_mac[3], s_mac[4], s_mac[5]);

	GetGateway(srcip, sgatewayip, &gatewayip);
	printf("\nSelected device has gateway : %s", sgatewayip);
	destip.s_addr = gatewayip;

	GetMacAddress(d_mac, destip);
	printf(" (Mac : %.2X-%.2X-%.2X-%.2X-%.2X-%.2X)", d_mac[0], d_mac[1], d_mac[2], d_mac[3], d_mac[4], d_mac[5]);

	//Now open the selected device
	printf("\nOpening the selected device...");

	if ((fp = pcap_open(dev[count].name,        // name of the device
		100,						// portion of the packet to capture (only the first 100 bytes)
		PCAP_OPENFLAG_PROMISCUOUS,  // promiscuous mode
		1000,						// read timeout
		NULL,						// authentication on the remote machine
		errbuf						// error buffer
	)) == NULL)
	{
		fprintf(stderr, "\nUnable to open the adapter. %s is not supported by WinPcap\n", dev[count].name);
		return 1;
	}
	printf("Opened");
	pcap_freealldevs(alldevs);

	// *******************  Ethernet Header *****************

	ehdr = (PETHER_HDR)packet;

	memcpy(ehdr->source, s_mac, 6);	//Source Mac address
	memcpy(ehdr->dest, d_mac, 6);	//Destination MAC address
	ehdr->type = htons(0x0800); //IP Frames

	// *******************  IP Header *****************

	iphdr = (PIPV4_HDR)(packet + sizeof(ETHER_HDR));

	iphdr->ip_version = 4;
	iphdr->ip_header_len = 5;	//In double words thats 4 bytes
	iphdr->ip_tos = 0;
	iphdr->ip_total_length = htons(sizeof(IPV4_HDR) + sizeof(TCP_HDR) + strlen(dump));
	iphdr->ip_id = htons(2);
	iphdr->ip_frag_offset = 0;
	iphdr->ip_reserved_zero = 0;
	iphdr->ip_dont_fragment = 1;
	iphdr->ip_more_fragment = 0;
	iphdr->ip_frag_offset1 = 0;
	iphdr->ip_ttl = 3;
	iphdr->ip_protocol = IPPROTO_TCP;
	iphdr->ip_srcaddr = inet_addr(IP_SRC);   //srcip.s_addr;
	iphdr->ip_destaddr = inet_addr(IP_DEST);
	iphdr->ip_checksum = 0;
	iphdr->ip_checksum = in_checksum((unsigned short*)iphdr, sizeof(IPV4_HDR));

	// *******************  TCP Header *****************
	tcphdr = (PTCP_HDR)(packet + sizeof(ETHER_HDR) + sizeof(IPV4_HDR));

	tcphdr->source_port = htons(5000);
	tcphdr->dest_port = htons(80);
	tcphdr->sequence = 0;
	tcphdr->acknowledge = 0;
	tcphdr->reserved_part1 = 0;
	tcphdr->data_offset = 5;
	tcphdr->fin = 0;
	tcphdr->syn = 1;
	tcphdr->rst = 0;
	tcphdr->psh = 0;
	tcphdr->ack = 0;
	tcphdr->urg = 0;
	tcphdr->ecn = 0;
	tcphdr->cwr = 0;
	tcphdr->window = htons(64240);
	tcphdr->checksum = 0;
	tcphdr->urgent_pointer = 0;

	// *******************  Data Dump *****************
	data = (char*)(packet + sizeof(ETHER_HDR) + sizeof(IPV4_HDR) + sizeof(TCP_HDR));
	strcpy(data, dump);

	// *******************  Checksum calculation *****************
	pseudo_header.source_address = inet_addr(IP_SRC); //forge it >:) srcip.s_addr; 
	pseudo_header.dest_address = inet_addr(IP_DEST);
	pseudo_header.placeholder = 0;
	pseudo_header.protocol = IPPROTO_TCP;
	pseudo_header.tcp_length = htons(sizeof(TCP_HDR) + strlen(dump));
	memcpy(&pseudo_header.tcp, tcphdr, sizeof(TCP_HDR));

	unsigned char* seudo;
	seudo = new unsigned char(sizeof(P_HDR) + strlen(dump));
	memcpy(seudo, &pseudo_header, sizeof(P_HDR));
	memcpy(seudo + sizeof(P_HDR), data, strlen(dump));

	tcphdr->checksum = in_checksum((unsigned short*)seudo, sizeof(P_HDR) + strlen(dump));

	printf("\nSending Packet...");

	//Uncomment this line if you want to flood
	//while(1) 
	{
		pcap_sendpacket(fp, packet, sizeof(ETHER_HDR) + sizeof(IPV4_HDR) + sizeof(TCP_HDR) + strlen(dump));
		printf("Send\n");
	}

	return 0;
}

in_addr getIPAddr(pcap_if_t* d) {
	pcap_addr_t* a;
	in_addr ia={0};
	for (a = d->addresses; a; a = a->next)	//Now print the IP addresses etc of each device
		if (a->addr->sa_family == AF_INET)
			if (a->addr)
				return ((struct sockaddr_in*)a->addr)->sin_addr;
				
	return ia;
}
/*
	Print all the available information about a winpcap device like lo , eth0
*/
void ifprint(pcap_if_t* d)
{
	pcap_addr_t* a;

	printf("%s\n", d->name);	//Name

	if (d->description)
	{
		printf("Description: %s\n", d->description);	//Description
	}

	// Loopback Address
	printf("Loopback: %s\n", (d->flags & PCAP_IF_LOOPBACK) ? "Yes" : "No");

	for (a = d->addresses; a; a = a->next)	//Now print the IP addresses etc of each device
	{
		printf("Address Family: #%d\n", a->addr->sa_family);

		switch (a->addr->sa_family)
		{
		case AF_INET:
			printf("Address Family Name: AF_INET\n");

			if (a->addr)
			{
				printf("Address: %s\n", inet_ntoa(((struct sockaddr_in*)a->addr)->sin_addr));
			}

			if (a->netmask)
			{
				//If a valid netmask has been detected
				printf("Netmask: %s\n", inet_ntoa(((struct sockaddr_in*)a->netmask)->sin_addr));
			}

			if (a->broadaddr)
			{
				//If a valid Broadcast Address is detected
				printf("Broadcast Address: %s\n", inet_ntoa(((struct sockaddr_in*)a->broadaddr)->sin_addr));
			}

			if (a->dstaddr)
			{
				printf("Destination Address: %s\n", inet_ntoa(((struct sockaddr_in*)a->dstaddr)->sin_addr));
			}
			break;

		default:
			printf("Address Family Name: Unknown\n");
			break;
		}
	}
	printf("\n");
}

/*
	General Networking Functions
	Checksum function - used to calculate the IP header and TCP header checksums
*/
unsigned short in_checksum(unsigned short* ptr, int nbytes)
{
	register long sum;
	unsigned short oddbyte;
	register short answer;

	sum = 0;
	while (nbytes > 1) {
		sum += *ptr++;
		nbytes -= 2;
	}
	if (nbytes == 1) {
		oddbyte = 0;
		*((u_char*)&oddbyte) = *(u_char*)ptr;
		sum += oddbyte;
	}

	sum = (sum >> 16) + (sum & 0xffff);
	sum = sum + (sum >> 16);
	answer = (SHORT)~sum;

	return(answer);
}

/*
	Initialise Winsock with the WSAStartup function
*/
int init_winsock()
{
	WSADATA firstsock;

	if (WSAStartup(MAKEWORD(2, 2), &firstsock) != 0)
	{
		printf("\nFailed to initialise winsock.");
		printf("\nError Code : %d", WSAGetLastError());
		return 1;	//Return 1 on error
	}

	return 0;	//Return 0 on successful
}